"""
Little script to lint the different pipelines by using GitLab CI Lint API:
https://docs.gitlab.com/ee/api/lint.html#validate-a-ci-yaml-configuration-with-a-namespace.

You can ignore a file by adding a comment "# ci_lint_ignore" anywhere in the file.

Requirements: Python 3.9 and requests
"""

import glob
import os
import sys
from typing import List, Iterator

import requests
from requests import HTTPError

# GitLab project ID of the repository containing the templates.
TEMPLATE_PROJECT_ID = os.environ["CI_PROJECT_ID"]
# GitLab API Token that has access to the project containing the templates
API_TOKEN = os.environ["GITLAB_TOKEN"]

# The files containing this comment will be ignored
IGNORE_LINT_MSG = "# ci_lint_ignore"


# https://stackoverflow.com/a/287944
class Colors:
    OKGREEN = '\033[92m'
    WARNING = '\033[93m'
    FAIL = '\033[91m'
    ENDC = '\033[0m'
    BOLD = '\033[1m'


def find_yml_files(root_path: str) -> Iterator[str]:
    """
    Find all yaml files in a specific folder
    :return:
    """
    return glob.iglob(os.path.join(root_path, "*.yml"), recursive=True)


def _get_errors(lint_errors: List[str]) -> str:
    """
    Return a string with all the errors
    :return:
    """
    errors_msg = f"{Colors.FAIL}\tErrors: \n"
    for err in lint_errors:
        errors_msg += f"\t\t - {err}"

    errors_msg += f"{Colors.ENDC}"
    return errors_msg


def _get_warnings(lint_warnings: List[str]) -> str:
    """
    Return a string with all the warnings
    :return:
    """
    warnings_msg = f"{Colors.WARNING}\tWarnings: \n"
    for warn in lint_warnings:
        warnings_msg += f"\t\t - {warn}"

    warnings_msg += f"{Colors.ENDC}"
    return warnings_msg


def ci_lint_yaml(yaml_file_path) -> bool:
    """
    Validate a single yaml file
    :param yaml_file_path:
    :return:
    """
    url = f"https://gitlab.com/api/v4/projects/{TEMPLATE_PROJECT_ID}/ci/lint"
    headers = {
        "Content-Type": "application/json",
        "PRIVATE-TOKEN": API_TOKEN
    }

    with open(yaml_file_path) as pipeline_yaml:
        ci_content = pipeline_yaml.read()

    if IGNORE_LINT_MSG in ci_content:
        print(f"Skipping '{yaml_file_path}' due to ignore comment.")
        return True

    data = {
        "content": ci_content,
        "include_jobs": True
    }
    response = requests.post(url, json=data, headers=headers)
    response.raise_for_status()
    json_response = response.json()
    errors = json_response["errors"]
    warnings = json_response["warnings"]

    if json_response["valid"]:
        print(f"{Colors.OKGREEN}Pipeline '{yaml_file_path}' is valid. {Colors.ENDC}")
    else:
        print(f"{Colors.FAIL}Pipeline '{yaml_file_path}' is invalid: {Colors.ENDC}")
        print(_get_errors(errors))

    if len(warnings) > 0:
        print(_get_warnings(warnings))

    return json_response["valid"]


if __name__ == '__main__':
    root_dir = "pipelines/**"
    print(f"Finding all *.yml files in '{root_dir}'")

    results = []

    for f in find_yml_files(root_dir):
        try:
            results.append(ci_lint_yaml(f))
        except HTTPError as e:
            print(f"{Colors.FAIL}HTTPError {e.response.status_code} when linting '{f}': {e.response.text}{Colors.ENDC}")

    if not all(results):
        # There is at least one error
        print(f"{Colors.FAIL}There were some errors. Exiting{Colors.ENDC}")
        sys.exit(1)
    else:
        print(f"Success. Everything is fine")
        sys.exit(0)
